#!/usr/bin/perl -w

# Really useful when script is run from the relative path
BEGIN { unshift @INC, $1 if $0 =~ /(.*)[\/]/; }

use strict;
use klinstall;
use appdata;


############################## text strings ##################################
my $UNINSTALL_MSG_1 = <<UNINSTALL_MSG_1_END;
%1 is not installed or has already been uninstalled.
UNINSTALL_MSG_1_END

my $UNINSTALL_MSG_2 = <<UNINSTALL_MSG_2_END;
%1 is required for %2 to work!
UNINSTALL_MSG_2_END

my $UNINSTALL_MSG_3 = <<UNINSTALL_MSG_3_END;
This script should not be run by the user - use your package manager to uninstall the software.
UNINSTALL_MSG_3_END

my $UNINSTALL_MSG_4 = <<UNINSTALL_MSG_4_END;
installation database is damaged, uninstall will be incompleted.
UNINSTALL_MSG_4_END

my $UNINSTALL_MSG_5 = <<UNINSTALL_MSG_5_END;
%1 has been uninstalled successfully.
UNINSTALL_MSG_5_END


############################### main() ######################################

	klinstall::check_for_root ("uninstall script");

	my ($logic, $pkgmgr) = klinstall::get_work_logic($0, \@ARGV);

	# parse cmdline
	my $upgrade = 0;
	foreach my $opt (@ARGV) {
	    if ($opt =~ /^--upgrade(=(\d))?$/) {
	        $upgrade = !defined($2) || ($2 ne '0') ? 1 : 0;
	    }
	}

	# Get application info
	my %APPINFO = appdata::get_application_info();

	# get app info
	my %INSTINFO = klinstall::read_app_info ($APPINFO{"ID"});
	if ( !%INSTINFO )
	{
		# for .tar.gz - abort deinstallation
		klinstall::fatal ($UNINSTALL_MSG_1, $APPINFO{"NAME"}) if !$pkgmgr;

		# for rpm, etc - generate a warning and continue uninstall
		klinstall::warning ($UNINSTALL_MSG_4);
		exit 0;
	}

	# wrong package manager?
	klinstall::fatal ($UNINSTALL_MSG_3) if $INSTINFO{"USED_PKGMGR"} && !$pkgmgr;

	# Check for dependend components
	if ( $INSTINFO{"DEPENDON"} )
	{
		foreach (split (/\s+/, $INSTINFO{"DEPENDON"}))
		{
			my %app = klinstall::read_app_info ($_);
			klinstall::fatal ($UNINSTALL_MSG_2, $APPINFO{"NAME"}, $app{"NAME"}) if !%app;
		}
	}

	# Custom pre-uninstall checks
	my $text = appdata::app_checks_preuninstall();
	klinstall::fatal ($text) if $text;

	# For rpm: rpm -U calls %postun for every updated packet. If it's just update, 
	# we probably don't need to remove AV bases, etc.
	if ( $pkgmgr eq 'rpm' && $INSTINFO{"IS_UPDATE"} )
	{
		$INSTINFO{"IS_UPDATE"} = 0;
		klinstall::write_app_info ($APPINFO{"ID"}, %INSTINFO);
		$upgrade = 1;
	}

	# Application-specific uninstall
	appdata::app_unregister($upgrade);

	# On upgrade we probably don't need to remove AV bases, etc.
	exit 0 if $upgrade;

	# remove files
	if ( $INSTINFO{"FILE"} )
	{
		if ( ref $INSTINFO{"FILE"} ne "ARRAY" )
		{
			unlink $INSTINFO{"FILE"} or klinstall::warning ("Could not unlink file $INSTINFO{FILE}: $!");
		}
		else
		{
			foreach my $file (@{$INSTINFO{"FILE"}})
			{
				$file = "$INSTINFO{$1}/$2" if $file =~ /^(\w+)\/(.*)$/ && defined $INSTINFO{$1};
				next if !-f $file;
				unlink $file or klinstall::warning ("Could not unlink file $file: $!");
			}
		}
	}

	# remove dirs in reversed order
	if ( $INSTINFO{"DIR"} )
	{
		foreach my $dir (reverse @{$INSTINFO{"DIR"}})
		{
			$dir = "$INSTINFO{$1}/$2" if $dir =~ /^(\w+)\/(.*)$/ && defined $INSTINFO{$1};
			next if !-d $dir;
			rmdir ($dir) or klinstall::warning ("Could not remove dir $dir: $!");
		}
	}

	# remove AV bases from BasesPath
	# on FreeBSD bases are keeped for package update
        my $PATH_BASES = $INSTINFO{"PATH_BASES"};
	if ( $PATH_BASES && ($^O !~ /^freebsd$/i) && opendir (DIR, $PATH_BASES) )
	{
	    while ( defined (my $entry = readdir (DIR)) )
	    {
		next if $entry =~ /^\.\.?$/;
		if (-d "$PATH_BASES/$entry")
		{
		    if ( opendir (SUBDIR, "$PATH_BASES/$entry") )
		    {
		    	while ( defined (my $subentry = readdir (SUBDIR)) )
			{
			    next if $entry =~ /^\.\.?$/ or !-f "$PATH_BASES/$entry/$subentry";
			    unlink ("$PATH_BASES/$entry/$subentry");
			}
		    }
		    closedir SUBDIR;

		    rmdir ("$PATH_BASES/$entry") or
			klinstall::warning ("Could not remove dir $PATH_BASES/$entry: $!");
		    next;
		}
		next if !-f "$PATH_BASES/$entry";

		unlink ("$PATH_BASES/$entry")
		    or klinstall::warning ("Could not remove file $PATH_BASES/$entry: $!");
		}

		closedir DIR;
		#rmdir ($PATH_BASES) or warning ("Could not remove dir $PATH_BASES: $!");
	}

	# remove the information from the storage
	klinstall::remove_app_info ($APPINFO{"ID"});
	
	klinstall::output ($UNINSTALL_MSG_5, $APPINFO{"NAME"}) if !$pkgmgr;
	exit 0;
